{ config, pkgs, lib, self, ... }:

let
  mainServer = "server1";
  pillar = self.lib.saltPillarFor mainServer;

  renameAttr = from: to: attrset:
    builtins.foldl' (result: name:
      if name == from
      then result // { "${to}" = attrset.${name}; }
      else result // { "${name}" = attrset.${name}; }
    ) {} (builtins.attrNames attrset);
in
{
  options.salt-pillar = lib.mkOption {};
  config.salt-pillar = pillar;

  config.site.net = lib.mkMerge ([
    (builtins.mapAttrs (_: vlan: { vlan = vlan; }) pillar.vlans)
    (builtins.mapAttrs (_: subnet4: { inherit subnet4; }) pillar.subnets-inet)
    (builtins.mapAttrs (_: hosts4: { inherit hosts4; }) pillar.hosts-inet)
  ] ++ (
    map (ctx:
      builtins.mapAttrs (_: subnet: { subnets6.${ctx} = subnet; }) pillar.subnets-inet6.${ctx}
    ) (builtins.attrNames pillar.subnets-inet6)
  ) ++ (
    map (ctx:
      builtins.mapAttrs (_: subnet: { hosts6.${ctx} = subnet; }) pillar.hosts-inet6.${ctx}
    ) (builtins.attrNames pillar.hosts-inet6)
  ));

  config.site.hosts = lib.mkMerge (
    [
      {
        "${mainServer}".role = "server";
      }

      (builtins.mapAttrs (_: switch: {
        inherit (switch) model location password;
        role = "switch";
      }) pillar.switches)

      (builtins.mapAttrs (_: ap: {
        inherit (ap) model location password;
        role = "ap";
      }) pillar.cpe)

      (builtins.mapAttrs (_: container: {
        role = "container";
        location = mainServer;
        interfaces =
          builtins.mapAttrs (_:
            renameAttr "gw" "gw6"
          ) container.interfaces;
      }) pillar.containers)
    ] ++

    (map (net:
      builtins.mapAttrs (_: addr4: {
      }) pillar.hosts-inet.${net}
    ) (builtins.attrNames pillar.hosts-inet)) ++

    (builtins.concatMap (ctx:
      map (net:
        builtins.mapAttrs (_: addr6: {
        }) pillar.hosts-inet6.${ctx}.${net}
      ) (builtins.attrNames pillar.hosts-inet6.${ctx})
    ) (builtins.attrNames pillar.hosts-inet6))
  );
}
